@@ -296,7 +296,7 @@ static CandidateController *sharedController;
 
   [window orderOut:nil];
 
-  (*_callBack)(kEventCandClicked);
+  (*_callBack)([sender clickedRow]);
 }
 
 - (void)replaceWindow:(int)replyX:(int)replyY
