@@ -69,3 +69,47 @@ static struct {
   {0x7e, UKey_Up},
   {NULL, NULL}
 };
+
+// convert control sequence to keycode
+static struct {
+  const unsigned char charcode;
+  const int ckey; // with <control>
+  //const int skey; // with <shift> + <control>
+} CharToKey[] = {
+  {0x1f, '-' /* , '=' */},
+  {0x36, '^' /* , '~' */},
+  {0x1c, '\\' /* , '|' */},
+  {0x32, '@' /* , '`' */},
+  {0x1b, '[' /* , '{' */},
+  {0x1d, ']' /* , '}' */},
+  //{0x1f, '_' /* , '_' */},
+  {NULL, NULL /* , NULL */}
+};
+
+#if 0
+// for Japanese key mapping
+static struct {
+  const unsigned char keycode;
+  const int ckey;
+  const int skey;
+} CSeqToKey[] = {
+  {0x12, 0, '!'},
+  {0x13, 0, '"'},
+  {0x14, 0, '#'},
+  {0x15, 0, '$'},
+  {0x17, 0, '%'},
+  {0x16, 0, '&'},
+  {0x1a, 0, '\''},
+  {0x1c, 0, '('},
+  {0x19, 0, ')'},
+  {0x1b, '-', '='},
+  {0x18, '^', '~'},
+  {0x5d, '\\', '|'},
+  {0x21, '@', '`'},
+  {0x1e, '[', '{'},
+  {0x29, ';', '+'},
+  {0x27, ':', '*'},
+  {0x2a, ']', '}'},
+  {NULL, NULL}
+};
+#endif
