@@ -48,18 +48,7 @@
 #include "Preference.h"
 
 
-#define ENABLE_HELPER  1
-
-//#define DEBUG_KEYEVENT  1
-//#define DEBUG_MENU  1
-//#define DEBUG_CANDIDATES  1
-//#define DEBUG_SESSION  1
-//#define DEBUG_UIM  "3"
-//#define DEBUG_PRIME  1
-
-
 const short kMENU_Pencil = kBaseResourceID + 1;
-
 const short kICNx_Pencil = kBaseResourceID + 1;
 
 enum
@@ -89,15 +78,12 @@ enum
 };
 
 MUIMSessionHandle gActiveSession;
-Boolean gDisableFocusedContext = FALSE;
 
 int gNumSession = 0;
 MUIMSessionHandle *gSessionList = NULL;
 
-uim_context gUC = NULL;
-
 #ifdef NEW_HELPER
-Boolean gUimIsHelperConnect;
+Boolean gUimHelperConnected;
 #else
 int gUimFD;
 #endif
@@ -105,6 +91,12 @@ CFSocketRef gUimSock;
 CFRunLoopSourceRef gUimRun;
 CFSocketContext gSockContext;
 
+#ifdef SYNC_MODE
+/** current mode */
+SInt32 gMode = -1;
+#endif
+CFStringRef gIMName = NULL;
+
 static MenuRef gPencilMenu;
 
 static CFStringRef gShowKeyboardPaletteMenuItemString;
@@ -117,10 +109,13 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
                            EventRef inEventRef, void *inUserData);
 
 static void
-CreateUIMContext(CFStringRef inIMName);
+CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName);
 
 static void
-ReleaseUIMContext();
+CreateAllUIMContext(CFStringRef inIMName);
+
+static void
+ReleaseAllUIMContext();
 
 static void
 NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver, 
@@ -150,7 +145,6 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 
   gActiveSession = NULL;
   gPencilMenu = NULL;
-  gDisableFocusedContext = FALSE;
 
   refNum = OpenComponentResFile((Component) inComponentInstance);
   result = ResError();
@@ -279,14 +273,15 @@ MUIMInitialize(ComponentInstance inComponentInstance,
   uim_init();
 
 #ifdef NEW_HELPER
-  gUimIsHelperConnect = FALSE;
+  gUimHelperConnected = FALSE;
 #else
   gUimFD = -1;
 #endif
+
   gUimSock = NULL;
   gUimRun = NULL;
 
-  CreateUIMContext(NULL);
+  //CreateUIMContext(NULL);
 
   {
     CFNotificationCenterRef center =
@@ -320,7 +315,6 @@ MUIMTerminate(ComponentInstance inComponentInstance)
     uim_quit();
 
     gActiveSession = NULL;
-    gDisableFocusedContext = FALSE;
     gPencilMenu = NULL;
 }
 
@@ -357,16 +351,20 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
     (**outSessionHandle)->fSegments = NULL;
     (**outSessionHandle)->fSegmentCount = 0;
 
-    //(**outSessionHandle)->fUC = gUC;
-
     (**outSessionHandle)->fLastUpdateLength = 0;
     (**outSessionHandle)->fFixBuffer = NULL;
     (**outSessionHandle)->fFixLen = 0;
     (**outSessionHandle)->fOldPreedit = NULL;
     (**outSessionHandle)->fOldPreeditLen = 0;
 
+    //(**outSessionHandle)->fMode = -1;
+    //(**outSessionHandle)->fModeLock = false;
+
     (**outSessionHandle)->fBundleRef = NULL;
     (**outSessionHandle)->fWindowOpened = false;
+
+    (**outSessionHandle)->fUC = NULL;
+    CreateUIMContext(*outSessionHandle, NULL);
   }
   else
     result = memFullErr;
@@ -421,24 +419,54 @@ ComponentResult
 MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 {
   OSStatus result = noErr;
-
-  gActiveSession = inSessionHandle;
-  gDisableFocusedContext = FALSE;
-
   long keyboardID = GetScriptVariable(GetScriptManagerVariable(smLastScript),
                                       smScriptKeys);
-  SetScriptVariable (smJapanese, smScriptKeys, keyboardID);
+  SetScriptVariable(smJapanese, smScriptKeys, keyboardID);
+
+  gActiveSession = inSessionHandle;
 
 #if DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p\n",
               gActiveSession);
 #endif
 
+  if ((*inSessionHandle)->fWindowOpened)
+    LayoutCandidate(inSessionHandle);
+
+#ifdef SYNC_MODE
+#if DEBUG_HELPER
+  DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p mode=%d gMode=%d fSegmentCount=%lu\n",
+              gActiveSession,
+              uim_get_current_mode((*inSessionHandle)->fUC), gMode,
+              (*inSessionHandle)->fSegmentCount);
+#endif
+#if ENABLE_HELPER
+  if (gMode < 0) {
+    UIMCheckHelper();
+    uim_prop_list_update((*inSessionHandle)->fUC);
+    uim_prop_label_update((*inSessionHandle)->fUC);
+    //uim_helper_client_focus_in((*inSessionHandle)->fUC);
+  }
+#endif // ENABLE_HELPER
+
+  if (gMode >= 0
+      && gMode != uim_get_current_mode((*inSessionHandle)->fUC)
+      // && (*inSessionHandle)->fSegmentCount <= 1
+      ) {
+#if DEBUG_HELPER
+    DEBUG_PRINT("MUIMSessionActivate() set mode %d\n", gMode);
+#endif
+    uim_set_mode((*inSessionHandle)->fUC, gMode);
+  }
+#endif // SYNC_MODE
+
+#ifndef SYNC_MODE
 #if ENABLE_HELPER
   UIMCheckHelper();
-  uim_prop_list_update(gUC);
-  uim_prop_label_update(gUC);
-  uim_helper_client_focus_in(gUC);
+  uim_prop_list_update((*inSessionHandle)->fUC);
+  uim_prop_label_update((*inSessionHandle)->fUC);
+  //uim_helper_client_focus_in((*inSessionHandle)->fUC);
+#endif
 #endif
 
   return result;
@@ -452,15 +480,11 @@ MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
               gActiveSession);
 #endif
 
-  // commit preedit string (multi context)
-  MUIMSessionFix(inSessionHandle);
+  MUIMHideWindow(inSessionHandle);
 
-  /*
-  if (inSessionHandle == gActiveSession) {
-    gActiveSession = nil;
-    gDisableFocusedContext = TRUE;
-  }
-  */
+#if ENABLE_HELPER
+  //uim_helper_client_focus_out((*inSessionHandle)->fUC);
+#endif
 
   return noErr;
 }
@@ -477,15 +501,16 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
   eventClass = GetEventClass(inEventRef);
   eventKind = GetEventKind(inEventRef);
 
+#if DEBUG_SESSION
   // kEventRawKeyDown = 1
   // kEventRawKeyRepeat = 2
   // kEventRawKeyUp = 3
   // kEventRawKeyModifiersChanged = 4
   // kEventHotKeyPressed = 5
   // kEventHotKeyReleased = 6
-
-  //DEBUG_PRINT("MUIMSessionEvent() eventClass=%lu eventKind=%lu\n",
-  //            eventClass, eventKind);
+  DEBUG_PRINT("MUIMSessionEvent() eventClass=%lu eventKind=%lu\n",
+              eventClass, eventKind);
+#endif
 
   // kEventClassKeyboard:
   //   kEventRawKeyDown - A key was pressed
@@ -535,6 +560,7 @@ MUIMSessionFix(MUIMSessionHandle inSessionHandle)
     // commit preedit string (single context)
     GetPreeditString(inSessionHandle, &str, &len, &cursorPos);
     if (len > 0) {
+#if 0
       (*inSessionHandle)->fFixBuffer = str;
       (*inSessionHandle)->fFixLen = len;
 
@@ -544,18 +570,33 @@ MUIMSessionFix(MUIMSessionHandle inSessionHandle)
       (*inSessionHandle)->fFixBuffer = NULL;
       (*inSessionHandle)->fFixLen = 0;
 
-      uim_reset_context(gUC);
+      uim_reset_context((*inSessionHandle)->fUC);
+#else
+      uim_press_key((*inSessionHandle)->fUC, 'j', UMod_Control);
+      uim_release_key((*inSessionHandle)->fUC, 'j', UMod_Control);
+#endif
     }
 
 #if ENABLE_HELPER
-    if ((*inSessionHandle)->fWindowOpened)
-      HideCandidateWindow(inSessionHandle);
+    MUIMHideWindow(inSessionHandle);
 #endif
   }
 
   return noErr;
 }
 
+ComponentResult
+MUIMHideWindow(MUIMSessionHandle inSessionHandle)
+{
+#if DEBUG_SESSION
+  DEBUG_PRINT("MUIMHidePaletteWindow()\n");
+#endif
+
+  HideCandidateWindow(inSessionHandle);
+
+  return noErr;
+}
+
 #pragma mark -
 
 MUIMSessionHandle
@@ -612,15 +653,16 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
     }
   }
 
-
   mod = ConvertModifier(inModifiers);
 
 #if DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() key=0x%x mod=0x%x\n", key, mod);
 #endif
 
-  rv = uim_press_key(gUC, key, mod);
-  uim_release_key(gUC, key, mod);
+  rv = uim_press_key((*inSessionHandle)->fUC, key, mod);
+
+  uim_release_key((*inSessionHandle)->fUC, key, mod);
+
   if (!rv || isScriptKey)
     handled = TRUE;
 
@@ -710,7 +752,7 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
 }
 
 static void
-CreateUIMContext(CFStringRef inIMName)
+CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName)
 {
   const char *imName;
   CFPropertyListRef imVal = NULL;
@@ -718,6 +760,9 @@ CreateUIMContext(CFStringRef inIMName)
   if (inIMName) {
     imName = CFStringGetCStringPtr(inIMName, kCFStringEncodingMacRoman);
   }
+  else if (gIMName) {
+    imName = CFStringGetCStringPtr(gIMName, kCFStringEncodingMacRoman);
+  }
   else {
     // Load the preferences
     imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), CFSTR(kAppID));
@@ -728,42 +773,66 @@ CreateUIMContext(CFStringRef inIMName)
       imName = kDefaultIM;
   }
 
-  //DEBUG_PRINT("CreateUIMContext() imName='%s'\n", imName);
-
-  gUC = uim_create_context(gSessionList, "UTF-8", NULL, imName,
-                           uim_iconv, UIMCommitString);
+  (*inSessionHandle)->fUC =
+    uim_create_context(gSessionList, "UTF-8", NULL, imName,
+                       uim_iconv, UIMCommitString);
 
   if (imVal)
     CFRelease(imVal);
 
-  if (!gUC)
+  if (!(*inSessionHandle)->fUC)
     return;
 
-  uim_set_preedit_cb(gUC,
+  uim_set_preedit_cb((*inSessionHandle)->fUC,
                      UIMPreeditClear,
                      UIMPreeditPushback,
                      UIMPreeditUpdate);
 
 #if ENABLE_HELPER
   UIMCheckHelper();
-  uim_set_prop_list_update_cb(gUC, UIMUpdatePropList);
-  uim_set_prop_label_update_cb(gUC, UIMUpdatePropLabel);
-  uim_prop_list_update(gUC);
-  uim_prop_label_update(gUC);
-#endif
+  uim_set_prop_list_update_cb((*inSessionHandle)->fUC, UIMUpdatePropList);
+  uim_set_prop_label_update_cb((*inSessionHandle)->fUC, UIMUpdatePropLabel);
+#ifndef SYNC_MODE
+  uim_prop_list_update((*inSessionHandle)->fUC);
+  uim_prop_label_update((*inSessionHandle)->fUC);
+#endif // SYNC_MODE
+#endif // ENABLE_HELPER
 
-  uim_set_candidate_selector_cb(gUC,
+  uim_set_candidate_selector_cb((*inSessionHandle)->fUC,
                                 UIMCandAcivate,
                                 UIMCandSelect,
                                 UIMCandShiftPage,
                                 UIMCandDeactivate);
+
+#ifdef SYNC_MODE
+  uim_set_mode_cb((*inSessionHandle)->fUC,
+                  UIMModeUpdate);
+#endif
 }
 
 static void
-ReleaseUIMContext()
+CreateAllUIMContext(CFStringRef inIMName)
 {
-  uim_release_context(gUC);
-  gUC = NULL;
+  UInt32 i;
+
+  for (i = 0; i < gNumSession; i++)
+    CreateUIMContext(gSessionList[i], inIMName);
+}
+
+static void
+ReleaseAllUIMContext()
+{
+  UInt32 i;
+
+  for (i = 0; i < gNumSession; i++) {
+    MUIMSessionFix(gSessionList[i]);
+
+    HideCandidateWindow(gSessionList[i]);
+    (*gSessionList[i])->fWindowOpened = false;
+
+    uim_release_context((*gSessionList[i])->fUC);
+    (*gSessionList[i])->fUC = NULL;
+  }
 }
 
 static void
@@ -774,9 +843,13 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
   CFStringRef *keys;
   CFStringRef *values;
   int count;
+  UInt32 i;
 
   count = CFDictionaryGetCount(inUserInfo);
-  //DEBUG_PRINT("NotificationCallback() count=%d\n", count);
+
+#if DEBUG_NOTIFY
+  DEBUG_PRINT("NotificationCallback() count=%d\n", count);
+#endif
 
   if (count > 0) {
     keys = (CFStringRef *) malloc(count * sizeof(CFStringRef));
@@ -785,12 +858,25 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
     CFDictionaryGetKeysAndValues(inUserInfo,
                                  (const void **) keys, (const void **) values);
 
-    //DEBUG_PRINT("NotificationCallback() keys[0]='%s' values[0]='%s'\n",
-    //            CFStringGetCStringPtr(keys[0], kCFStringEncodingASCII),
-    //            CFStringGetCStringPtr(values[0], kCFStringEncodingASCII));
+#if DEBUG_NOTIFY
+    DEBUG_PRINT("NotificationCallback() keys[0]='%s' values[0]='%s'\n",
+                CFStringGetCStringPtr(keys[0], kCFStringEncodingASCII),
+                CFStringGetCStringPtr(values[0], kCFStringEncodingASCII));
+#endif
 
-    ReleaseUIMContext();
-    CreateUIMContext(values[0]);
+    if (gIMName)
+      CFRelease(gIMName);
+    gIMName = values[0];
+    CFRetain(gIMName);
+
+    ReleaseAllUIMContext();
+    CreateAllUIMContext(values[0]);
+
+    gMode = -1;
+    for (i = 0; i < gNumSession; i++) {
+      uim_prop_list_update((*gSessionList[i])->fUC);
+      uim_prop_label_update((*gSessionList[i])->fUC);
+    }
 
     free(keys);
     free(values);
@@ -801,7 +887,6 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
  * key codes are defined in
  * /System/Library/Frameworks/Carbon.framework/Frameworks/HIToolbox.framework/Headers/Events.h
  */
-
 static int
 ConvertKeyVal(int inKey, int inMod)
 {
