@@ -42,38 +42,81 @@
 #include "Utils.h"
 #include "Debug.h"
 
+#define ENABLE_HELPER  1
+#define SYNC_MODE  1
+
+/**
+ * preedit segment
+ */
 typedef struct _PreeditSegment
 {
+  /** segment buffer */
   UniCharPtr fBuffer;
+
+  /** segment length */
   UInt32 fLength;
+
+  /** segment attributes */
   UInt32 fAttr;
 
 } PreeditSegment;
 
+/**
+ * MacUIM session record
+ */
 typedef struct _MUIMSessionRecord
 {
+  /** component instance */
   ComponentInstance fComponentInstance;
+
+  /** preedit segment array */
   PreeditSegment *fSegments;
+
+  /** preedit segment count */
   UInt32 fSegmentCount;
 
-  //uim_context fUC;
+  /** Uim context */
+  uim_context fUC;
 
+  /** last updated length */
   UInt32 fLastUpdateLength;
     
+  /** current preedit buffer */
   UniCharPtr fFixBuffer;
+
+  /** current preedit length */
   UInt32 fFixLen;
 
+  /** old preedit buffer */
   UniCharPtr fOldPreedit;
+
+  /** old preedit length */
   UInt32 fOldPreeditLen;
 
+  /** total number of canddidates */
   SInt32 fNRCandidates;
-  SInt32 fCandidateIndex;
-  SInt32 fLayoutBegin;
-  UInt32 fIsActive;
+
+  /** maximum number of candidate display */
   SInt32 fDisplayLimit;
+
+  /** candidate index */
+  SInt32 fCandidateIndex;
+
+  /** page index */
+  SInt32 fPageIndex;
+
+  /** beginning index of displayed candidates */
+  SInt32 fLayoutBegin;
+
+  /** candidate window is active */
+  UInt32 fIsActive;
+
   SInt32 fSelectCount;
 
+  /** bundle reference of candidate window */
   CFBundleRef fBundleRef;
+
+  /** open candidate window when the context is activated */
   Boolean fWindowOpened;
 
 } MUIMSessionRecord;
@@ -81,6 +124,9 @@ typedef struct _MUIMSessionRecord
 typedef MUIMSessionRecord *MUIMSessionPtr;
 typedef MUIMSessionPtr *MUIMSessionHandle;
 
+/**
+ * TSM status
+ */
 typedef struct _TSMStat {
   long fRefCon;
   long fTextOffset;
@@ -117,10 +163,11 @@ MUIMSessionDeactivate(MUIMSessionHandle sessionHandle);
 ComponentResult
 MUIMSessionEvent(MUIMSessionHandle sessionHandle, EventRef eventRef);
 
-/*
 ComponentResult
 MUIMSessionFix(MUIMSessionHandle sessionHandle);
-*/
+
+ComponentResult
+MUIMHideWindow(MUIMSessionHandle inSessionHandle);
 
 MUIMSessionHandle
 MUIMGetActiveSession(void);
@@ -137,7 +184,7 @@ GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
                  UInt32 *outLen, UInt32 *outCursorPos);
 
 void
-SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange);
+CandClicked(UInt32 inRow);
 
 void
 DumpString(const char *name, const char *str, int len);
