@@ -35,6 +35,7 @@
 #import <sys/socket.h>
 #import <iconv.h>
 #import "MacUIMHelperExtra.h"
+#import "Debug.h"
 
 
 static MacUIMHelperExtra *sharedExtra;
@@ -361,6 +362,9 @@ convertHelperString(char *str);
   uim_helper_read_proc(uimFD);
 #endif
   while ((tmp = uim_helper_get_message())) {
+#if DEBUG_HELPER_EXTRA
+    fprintf(stderr, "MacUIMHelperExtra::helperRead() tmp='%s'\n", tmp);
+#endif
     [self helperParse:tmp];
     free(tmp);
   }
@@ -416,6 +420,12 @@ convertHelperString(char *str);
           [labels removeAllObjects];
         }
         */
+#if DEBUG_HELPER_EXTRA
+        else {
+          fprintf(stderr, "MacUIMHelperExtra::helperParse() unknown string '%s'\n",
+                  [first cString]);
+        }
+#endif
       }
     }
     [nsstr release];
@@ -538,7 +548,9 @@ convertHelperString(char *str);
     if (cols && [cols count] >= 2) {
       NSMutableString *label = [[NSMutableString alloc]
                                  initWithString:[cols objectAtIndex:0]];
-      //printf("propLabelUpdate: label='%s'\n", [label UTF8String]);
+#if DEBUG_HELPER_EXTRA
+      fprintf(stderr, "propLabelUpdate: label='%s'\n", [label UTF8String]);
+#endif
       [labels addObject:label];
     }
   }
