@@ -40,15 +40,15 @@
 #include "CandidateCarbon.h"
 
 extern MUIMSessionHandle gActiveSession;
-extern Boolean gDisableFocusedContext;
-
 extern int gNumSession;
 extern MUIMSessionHandle *gSessionList;
 
-extern uim_context gUC;
+#ifdef SYNC_MODE
+extern SInt32 gMode;
+#endif
 
 #ifdef NEW_HELPER
-extern int gUimIsHelperConnect;
+extern Boolean gUimHelperConnected;
 #else
 extern int gUimFD;
 #endif
@@ -56,7 +56,6 @@ extern CFSocketRef gUimSock;
 extern CFRunLoopSourceRef gUimRun;
 extern CFSocketContext gSockContext;
 
-
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr);
@@ -185,11 +184,14 @@ UIMCandAcivate(void *inPtr, int inNR, int inLimit)
 
   (*handle)->fIsActive = true;
   (*handle)->fCandidateIndex = -1;
+  (*handle)->fPageIndex = -1;
   (*handle)->fNRCandidates = inNR;
   (*handle)->fDisplayLimit = inLimit;
   (*handle)->fLayoutBegin = 0;
 
   LayoutCandidate(handle);
+
+  (*handle)->fWindowOpened = true;
 }
 
 /**
@@ -206,6 +208,7 @@ UIMCandSelect(void *inPtr, int inIndex)
 #endif
 
   (*handle)->fCandidateIndex = inIndex;
+  (*handle)->fPageIndex = (*handle)->fCandidateIndex / (*handle)->fDisplayLimit;
 
   LayoutCandidate(handle);
 }
@@ -214,32 +217,46 @@ UIMCandSelect(void *inPtr, int inIndex)
  * Candidate window page shift callback
  */
 void
-UIMCandShiftPage(void *inPtr, int inDirection)
+UIMCandShiftPage(void *inPtr, int inForward)
 {
   //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   MUIMSessionHandle handle = gActiveSession;
 
-  if (inDirection) {
+  if (inForward) { // next page
     if ((*handle)->fNRCandidates <
-        (*handle)->fCandidateIndex + (*handle)->fDisplayLimit)
+        (*handle)->fCandidateIndex + (*handle)->fDisplayLimit) {
+      // if the next page is not exist
+      // move to top page and select first candidate
       (*handle)->fCandidateIndex = 0;
-    else
+      (*handle)->fPageIndex = 0;
+    }
+    else {
+      // move to next page
       (*handle)->fCandidateIndex += (*handle)->fDisplayLimit;
+      (*handle)->fPageIndex++;
+    }
   }
-  else {
-    if ((*handle)->fCandidateIndex - (*handle)->fDisplayLimit < 0)
+  else { // previous page
+    if ((*handle)->fCandidateIndex - (*handle)->fDisplayLimit < 0) {
+      // if the previous page is not exist
+      // move to buttom page and select last candidate
       (*handle)->fCandidateIndex = (*handle)->fNRCandidates - 1;
-    else
+      (*handle)->fPageIndex = (*handle)->fNRCandidates / (*handle)->fDisplayLimit
+        + (((*handle)->fNRCandidates % (*handle)->fDisplayLimit) ? 1 : 0) - 1;
+    }
+    else {
       (*handle)->fCandidateIndex -= (*handle)->fDisplayLimit;
+      (*handle)->fPageIndex--;
+    }
   }
 
 #if DEBUG_CANDIDATES
-  DEBUG_PRINT("UIMCandShiftPage() inDirection=%d fCandidateIndex=%ld\n",
-              inDirection, (*handle)->fCandidateIndex);
+  DEBUG_PRINT("UIMCandShiftPage() inForward=%d fCandidateIndex=%ld\n",
+              inForward, (*handle)->fCandidateIndex);
 #endif
 
   LayoutCandidate(handle);
-  uim_set_candidate_index(gUC, (*handle)->fCandidateIndex);
+  uim_set_candidate_index((*handle)->fUC, (*handle)->fCandidateIndex);
 }
 
 /**
@@ -256,6 +273,7 @@ UIMCandDeactivate(void *inPtr)
 #endif
 
   HideCandidateWindow(handle);
+  (*handle)->fWindowOpened = false;
 }
 
 void
@@ -275,7 +293,7 @@ UIMUpdatePropList(void *inPtr, const char *inStr)
                          strlen(inStr) + 1));
   if (tmp) {
 #ifdef NEW_HELPER
-    if (gUimIsHelperConnect)
+    if (gUimHelperConnected)
 #else
     if (gUimFD >= 0)
 #endif
@@ -310,7 +328,7 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
                          strlen(inStr) + 1));
   if (tmp) {
 #if NEW_HELPER
-    if (gUimIsHelperConnect)
+    if (gUimHelperConnected)
 #else
     if (gUimFD >= 0)
 #endif
@@ -332,15 +350,15 @@ void
 UIMCheckHelper()
 {
 #ifdef NEW_HELPER
-  if (!gUimIsHelperConnect) {
+  if (!gUimHelperConnected) {
     if (uim_helper_init_client(UIMHelperDisconnect) == 0) {
-      gUimIsHelperConnect = TRUE;
+      gUimHelperConnected = TRUE;
 
 #if DEBUG_HELPER
-      DEBUG_PRINT("UIMCheckHelper() gUimIsHelperConnect=%d\n", gUimIsHelperConnect);
+      DEBUG_PRINT("UIMCheckHelper() gUimHelperConnected=%d\n", gUimHelperConnected);
 #endif
 
-      if (gUimIsHelperConnect) {
+      if (gUimHelperConnected) {
         if (!gUimSock) {
           gSockContext.version = 0;
           //gSockContext.info = inHandle;
@@ -370,7 +388,9 @@ UIMCheckHelper()
       }
     }
   }
-#else
+
+#else // NEW_HELPER
+
   if (gUimFD < 0) {
     gUimFD = uim_helper_init_client_fd(UIMHelperDisconnect);
 
@@ -436,12 +456,14 @@ ParseHelperString(const char *str)
   DEBUG_PRINT("ParseHelperString() str='%s'\n", str);
 #endif
 
-  if (gActiveSession && !gDisableFocusedContext) {
+  if (gActiveSession) {
     if (strncmp("prop_list_get", str, 13) == 0) {
-      uim_prop_list_update(gUC);
+      for (i = 0; i < gNumSession; i++)
+        uim_prop_list_update((*gActiveSession)->fUC);
     }
     else if (strncmp("prop_label_get", str, 14) == 0) {
-      uim_prop_label_update(gUC);
+      for (i = 0; i < gNumSession; i++)
+        uim_prop_label_update((*gActiveSession)->fUC);
     }
     else if (strncmp("prop_activate", str, 13) == 0) {
       CFMutableStringRef cfstr;
@@ -463,20 +485,15 @@ ParseHelperString(const char *str)
                       CFStringGetCStringPtr(first, kCFStringEncodingUTF8),
                       CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
 #endif
-          for (i = 0; i < gNumSession; i++) {
-            if (*gSessionList[i])
-              uim_prop_activate(gUC,
-                                CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
-          }
+          for (i = 0; i < gNumSession; i++)
+            uim_prop_activate((*gSessionList[i])->fUC,
+                              CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
           CFRelease(second);
         }
         CFRelease(first);
       }
     }
     else if (strncmp("focus_in", str, 8) == 0) {
-      //gDisableFocusedContext = TRUE;
-      /* We shouldn't do "focused_context = NULL" here, because some
-         window manager has some focus related bugs. */
     }
   }
 }
@@ -497,7 +514,7 @@ UIMHelperDisconnect()
   gUimSock = NULL;
 
 #ifdef NEW_HELPER
-  gUimIsHelperConnect = FALSE;
+  gUimHelperConnected = FALSE;
 #else
   gUimFD = -1;
 #endif
@@ -511,7 +528,7 @@ UIMHelperClose()
 #endif
 
 #ifdef NEW_HELPER
-  if (gUimIsHelperConnect)
+  if (gUimHelperConnected)
     uim_helper_close_client();
 #else
   if (gUimFD >= 0)
@@ -519,6 +536,16 @@ UIMHelperClose()
 #endif
 }
 
+#ifdef SYNC_MODE
+void
+UIMModeUpdate(void *inPtr, int inMode)
+{
+  DEBUG_PRINT("UIMModeUpdate() inMode=%d\n", inMode);
+
+  gMode = inMode;
+}
+#endif
+
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr)
@@ -613,60 +640,19 @@ GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
 }
 
 void
-SegmentToHiliteRange(MUIMSessionHandle inHandle, TextRangeArrayPtr *inRange)
+CandClicked(UInt32 inRow)
 {
-  UInt32 i, pos = 0, j = 0;
+  UInt32 idx = 0;
 
-  (*inRange) = (TextRangeArrayPtr) NewPtrClear(sizeof(short) +
-                                               sizeof(TextRange) *
-                                               ((*inHandle)->fSegmentCount + 1));
+  idx = (*gActiveSession)->fDisplayLimit * (*gActiveSession)->fPageIndex + inRow;
 
-  /*
-   * Uim attribute:
-   *
-   * UPreeditAttr_None
-   * UPreeditAttr_UnderLine
-   * UPreeditAttr_Reverse
-   * UPreeditAttr_Cursor
-   * UPreeditAttr_Separator
-   */
+#if DEBUG_CANDIDATES
+  DEBUG_PRINT("CandClicked() inRow=%lu fPageIndex=%ld idx=%lu\n",
+              inRow, (*gActiveSession)->fPageIndex, idx);
+#endif
 
-  /*
-   * TSM HiliteStyle:
-   *
-   * 1: kCaretPosition
-   * 2: kRawText
-   * 3: kSelectedRawText
-   * 4: kConvertedText
-   * 5: kSelectedConvertedText
-   * 6: kBlockFillText  ?
-   * 7: kOutlineText  ?
-   * 8: kSelectedText
-   */
+  uim_set_candidate_index((*gActiveSession)->fUC, idx);
 
-  for (i = 0; i < (*inHandle)->fSegmentCount; i++) {
-    // cursor
-    // string length is 0 or greaer than 0
-    if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_Cursor) {
-      (*inRange)->fRange[j].fStart = (*inRange)->fRange[j].fEnd = pos * sizeof(UniChar);
-      (*inRange)->fRange[j].fHiliteStyle = kCaretPosition;
-      j++;
-    }
-
-    if ((*inHandle)->fSegments[i].fLength > 0) {
-      (*inRange)->fRange[j].fStart = pos * sizeof(UniChar);
-      pos += (*inHandle)->fSegments[i].fLength;
-      (*inRange)->fRange[j].fEnd = pos * sizeof(UniChar);
-
-      if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_Reverse)
-          (*inRange)->fRange[j].fHiliteStyle = kSelectedConvertedText;
-      else if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_UnderLine)
-        (*inRange)->fRange[j].fHiliteStyle = kConvertedText;
-      else
-        (*inRange)->fRange[j].fHiliteStyle = kRawText;
-      j++;
-    }
-  }
-
-  (*inRange)->fNumOfRanges = j;
+  uim_press_key((*gActiveSession)->fUC, 'j', UMod_Control);
+  uim_release_key((*gActiveSession)->fUC, 'j', UMod_Control);
 }
